from cricket.model import Project

class PyTestProject(Project):

    def discover_commandline(self):
        "Command line: Discover all available tests in a project."
        return ['python', '-m', 'cricket.unittest.discoverer']

    def execute_commandline(self, labels):
        "Return the command line to execute the specified test labels"
        return ['python', '-m', 'cricket.unittest.executor'] + labels


from __future__ import absolute_import

from django.conf import settings
from django.test.simple import DjangoTestSuiteRunner
from django.test.utils import get_runner

# Dynamically retrieve the test runner class for this project.
TestRunnerClass = get_runner(settings, None)


class TestDiscoverer(TestRunnerClass):
    """A Django test runner that prints out all the test that will be run.

    Doesn't actually run any of the tests.
    """
    def run_tests(self, test_labels, extra_tests=None, **kwargs):
        for test in self.build_suite([]):
            # Django 1.6 introduce the new-style test runner.
            # If that test runner is in use, we use the full test name.
            # If we're still using a pre 1.6-style runner, we need to
            # drop out all everything between the app name and the test module.
            if issubclass(TestRunnerClass, DjangoTestSuiteRunner):
                parts = test.id().split('.')
                tests_index = parts.index('tests')
                print '%s.%s.%s' % (parts[tests_index - 1], parts[-2], parts[-1])
            else:
                print test.id()

        return 0



# This is for backwards compatibility with Python version < 2.7 only
# for Python 2.7 and 3.x the default unittest is the correct package
# to use. unittest2 is a backport of this package to be used in
# versions < 2.7.
# Use
#       from cricket.compat import unittest
#
# to make versions work with all version of Python. This will be slowly
# deprecated in the future as Python < 2.7 becomes more and mor
# obsolete.
from __future__ import absolute_import
import unittest
if not hasattr(unittest.TestCase, 'assertIsNotNone'):
    import unittest2 as unittest



try:
    from idlelib.WidgetRedirector import WidgetRedirector
except ImportError:
    import platform
    import sys
    if platform.linux_distribution()[0] == 'Ubuntu':
        raise Exception("idlelib could not be found. " +
                        "You may need to install IDLE - try running " +
                        "'sudo apt-get install idle-python%s.%s'" % (
                            sys.version_info[0:2]
                        ))
    else:
        raise Exception("idlelib could not be found. " +
                        "Check your operating system instructions " +
                        "to work out how to install IDLE and idlelib.")

from Tkinter import *
from ttk import *


def tk_break(*args, **kwargs):
    "Return a Tk 'break' event result."
    return "break"


class ReadOnlyText(Text):
    """A Text widget that redirects the insert and delete
    handlers so that they are no-ops. This effectively makes
    the widget readonly with respect to keyboard input handlers.

    Adapted from http://tkinter.unpythonic.net/wiki/ReadOnlyText, which
    is itself adapting a solution described here: http://wiki.tcl.tk/1152
    """
    def __init__(self, *args, **kwargs):
        Text.__init__(self, *args, **kwargs)

        self.redirector = WidgetRedirector(self)
        self.insert = self.redirector.register("insert", tk_break)
        self.delete = self.redirector.register("delete", tk_break)



'''
The whole purpose of this module is to generate printed output.
It should be of the form:

'module.testcase.specifictest'
'module.testcase.specifictest2'
'module2.testcase.specifictest'

etc

Its primary API is the command-line, but it can
just as easily be called programmatically (see __main__)
'''

import unittest


def consume(iterable):
    input = list(iterable)
    while input:
        item = input.pop(0)
        try:
            data = iter(item)
            input = list(data) + input
        except:
            yield item

class PyTestDiscoverer:

    def __init__(self):

        self.collected_tests = []


    def __str__(self):
        '''
        Builds the dotted namespace expected by cricket
        '''

        resultstr = '\n'.join(self.collected_tests)

        return resultstr.strip()


    def collect_tests(self):
        '''
        Collect a list of potentially runnable tests
        '''
        
        loader = unittest.TestLoader()
        suite = loader.discover('.')
        flatresults = list(consume(suite))
        named = [r.id() for r in flatresults]
        self.collected_tests = named

            
if __name__ == '__main__':

    PTD = PyTestDiscoverer()
    PTD.collect_tests()
    print str(PTD)


'''
In general, you would expect that there would only be one project class
specified in this file. It provides the interface to executing test
collecetion and execution.
'''

from cricket.model import Project


class DjangoProject(Project):
    '''
    The Project is a wrapper around the command-line calls to interface
    to test collection and test execution
    '''

    def discover_commandline(self):
        "Command lineDiscover all available tests in a project."
        return ['python', 'manage.py', 'test', '--testrunner=cricket.django.discoverer.TestDiscoverer']

    def execute_commandline(self, labels):
        "Return the command line to execute the specified test labels"
        return ['python', 'manage.py', 'test', '--testrunner=cricket.django.executor.TestExecutor', '--noinput'] + labels



'''
This is the main entry point for running unittest test suites.
'''
from cricket.main import main
from cricket.unittest.model import PyTestProject

if __name__ == "__main__":
    main(PyTestProject)



'''
This is a thing which, when run, produces a stream
of well-formed test result outputs. Its processing is
initiated by the top-level Executor class.

Its main API is the command line, but it's just as sensible to
call into it. See __main__ for usage
'''

from cricket import pipes
import subprocess
import sys
import unittest

from cricket.pipes import PipedTestRunner

class PyTestExecutor:
    '''
    This is a thing which, when run, produces a stream
    of well-formed test result outputs. Its processing is
    initiated by the top-level Executor class
    '''

    def __init__(self):

        # Allows the executor to run a specified list of tests
        self.specified_list = None

    def run_only(self, specified_list):
        self.specified_list = specified_list

    def stream_suite(self, suite):

        pipes.PipedTestRunner().run(suite)

    def stream_results(self):        
        '''
        1.) Discover all tests if necessary
        2.) Otherwise fetch specific tests
        3.) Execute-and-stream
        '''

        loader = unittest.TestLoader()

        if not self.specified_list:
            suite = loader.discover('.')
            self.stream_suite(suite)

        else:

            for module in self.specified_list:
                suite = loader.loadTestsFromName(module)
                self.stream_suite(suite)


if __name__ == '__main__':

    run_only = None
    if len(sys.argv) > 1:
        run_only = sys.argv[1:]

    PTE = PyTestExecutor()

    if run_only:
        PTE.run_only(run_only)
    PTE.stream_results()


'''
The purpose of this module is to set up the Cricket GUI,
load a "project" for discovering and executing tests, and
to initiate the GUI main loop.
'''
from Tkinter import *

from cricket.view import (
    MainWindow,
    TestLoadErrorDialog,
    IgnorableTestLoadErrorDialog
)
from cricket.model import ModelLoadError


def main(Model):
    """Run the main loop of the app.

    Take the project Model as the argument. This model will be
    instantiated as part of the main loop.
    """
    # Set up the root Tk context
    root = Tk()

    # Construct an empty window
    view = MainWindow(root)

    # Try to load the project. If any error occurs during
    # project load, show an error dialog
    project = None
    while project is None:
        try:
            project = Model()
        except ModelLoadError as e:
            dialog = TestLoadErrorDialog(root, e.trace)
            if dialog.status == dialog.CANCEL:
                sys.exit(1)

    if project.errors:
        dialog = IgnorableTestLoadErrorDialog(root, '\n'.join(project.errors))
        if dialog.status == dialog.CANCEL:
            sys.exit(1)

    # Set the project for the main window.
    # This populates the tree, and sets listeners for
    # future tree modifications.
    view.project = project

    # Run the main loop
    try:
        view.mainloop()
    except KeyboardInterrupt:
        view.on_quit()
